@@ -52,6 +52,23 @@ void tangle_action_execute_full(TangleAction* action, GObject* source, const gch
 	}
 }
 
+void tangle_action_execute_full_list(TangleAction* action, GObject* source, const gchar* trigger, const gchar* first_property_name, ...) {
+	va_list properties_list;
+	
+	va_start(properties_list, trigger);
+	tangle_action_execute_full_valist(action, source, trigger, properties_list);
+	va_end(properties_list);
+}
+
+void tangle_action_execute_full_valist(TangleAction* action, GObject* source, const gchar* trigger, va_list properties_list) {
+	TangleProperties* properties;
+	
+	properties = tangle_properties_new();
+	tangle_properties_set_valist(properties, properties_list);
+	tangle_action_execute_full(action, source, trigger, properties);
+	g_object_unref(G_OBJECT(properties));
+}
+
 void tangle_action_set_next_action(TangleAction* action, TangleAction* next_action) {
 	if (action->priv->next_action) {
 		g_object_unref(action->priv->next_action);
