@@ -14,6 +14,7 @@
 #include "tangle-transition.h"
 #include "tangle-vault.h"
 #include "tangle-misc.h"
+#include "tangle-private.h"
 #include "marshalers.h"
 #include <gobject/gvaluecollector.h>
 #include <string.h>
@@ -34,6 +35,7 @@ G_DEFINE_ABSTRACT_TYPE_WITH_CODE(TangleActor, tangle_actor, CLUTTER_TYPE_ACTOR,
 
 struct _TangleActorPrivate {
 	TangleTemplate* template;
+	gboolean template_applied;
 
 	gfloat transition_move_x;
 	gfloat transition_move_y;
@@ -831,24 +833,8 @@ void tangle_actor_handle_descendant_event(TangleActor* actor, TangleEvent* event
 }
 
 gchar* tangle_actor_lookup_filename(TangleActor* actor, const gchar* filename) {
-	gchar* path = NULL;
-	gchar* dir;
-	
-	if (g_path_is_absolute(filename)) {
-		path = g_strdup(filename);
-	} else if (actor->priv->script) {
-		path = clutter_script_lookup_filename(actor->priv->script, filename);
-	} else {
-		dir = g_get_current_dir();
-		path = g_build_filename(dir, filename, NULL);
-		if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
-			g_free(path);
-			path = NULL;
-		}
-		g_free(dir);
-	}
-	
-	return path;
+
+	return _tangle_script_lookup_filename(actor->priv->script, filename);
 }
 
 ClutterScript* tangle_actor_get_script(TangleActor* actor) {
@@ -856,6 +842,15 @@ ClutterScript* tangle_actor_get_script(TangleActor* actor) {
 	return actor->priv->script;
 }
 
+void _tangle_actor_set_script(TangleActor* actor, ClutterScript* script) {
+	g_return_if_fail(TANGLE_IS_ACTOR(actor));
+	g_return_if_fail(CLUTTER_IS_SCRIPT(script));
+	g_return_if_fail(!actor->priv->script);
+	
+	actor->priv->script = script;
+	g_object_add_weak_pointer(G_OBJECT(actor->priv->script), (gpointer*)&actor->priv->script);
+}
+
 TangleSpacing* tangle_spacing_new(gfloat top, gfloat right, gfloat bottom, gfloat left) {
 	TangleSpacing* spacing;
 	
@@ -1297,7 +1292,7 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 	switch (prop_id) {
 		case PROP_TEMPLATE:
 			if (g_value_get_object(value) != NULL) {
-				g_assert(!actor->priv->template);
+				g_return_if_fail(!actor->priv->template);
 				actor->priv->template = TANGLE_TEMPLATE(g_value_get_object(value));
 				g_object_ref(actor->priv->template);
 				if (tangle_template_has_parameters(actor->priv->template)) {
@@ -1561,8 +1556,11 @@ static void tangle_actor_dispose(GObject* object) {
 		g_object_unref(actor->priv->hiding_timeline);
 	}
 	TANGLE_ACTOR_UNSET_FLAGS(actor, TANGLE_ACTOR_HIDING);
-
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->script);
+	
+	if (actor->priv->script) {
+		g_object_remove_weak_pointer(G_OBJECT(actor->priv->script), (gpointer*)&actor->priv->script);
+		actor->priv->script = NULL;
+	}
 
 	G_OBJECT_CLASS (tangle_actor_parent_class)->dispose (object);
 }
@@ -1974,94 +1972,30 @@ static void tangle_actor_init(TangleActor* actor) {
 }
 
 static gboolean tangle_actor_parse_custom_node(ClutterScriptable* scriptable, ClutterScript* script, GValue* value, const gchar* name, JsonNode* node) {
-	static TangleActor* previous_actor = NULL;
-
-	gboolean retvalue = FALSE;
+	gboolean retvalue;
 	TangleActor* actor;
-	gpointer type_class;
-	GParamSpec* param_spec;
-	const gchar* s;
-	GObject* object;
 
 	actor = TANGLE_ACTOR(scriptable);
-	g_assert(!actor->priv->script || actor->priv->script == script);
+	g_return_val_if_fail(!actor->priv->script || actor->priv->script == script, FALSE);
 
 	if (!actor->priv->script) {
-		/* TODO: This should be weak, should it not? */
-		g_object_ref(script);
-		actor->priv->script = script;
+		_tangle_actor_set_script(actor, script);
 	}
 
-	/* This is a hack to identify, when a new actor is being parsed and thus, when template parameters must be cleared. */
-	if (actor != previous_actor) {
-		previous_actor = actor;
-		if (actor->priv->template) {
-			tangle_template_clear_parameter_values(actor->priv->template);
-		}
-	}
-	
-	type_class = g_type_class_ref(G_OBJECT_TYPE(scriptable));
-	
-	if (name[0] == '$') {
-		if (!actor->priv->template) {
-			g_warning("A template parameter '%s' found, but no template for the object (id: %s), skipped.", name + 1, clutter_get_script_id(G_OBJECT(actor)));
-		} else if (tangle_template_get_parameter_value(actor->priv->template, name + 1)) {
-			g_warning("The value of a template parameter '%s' is defined twice for the object (id: %s), skipped.", name + 1, clutter_get_script_id(G_OBJECT(actor)));
-		} else if (!clutter_script_parse_node(script, value, name, node, NULL)) {
-			g_warning("The value of a template parameter '%s' could not be parsed (object: %s), skipped.", name + 1, clutter_get_script_id(G_OBJECT(actor)));
-		} else if (!tangle_template_set_parameter_value(actor->priv->template, name + 1, value)) {
-			g_warning("A template parameter '%s' does not exist in a template (id: %s) for the object (id: %s), skipped.", name + 1, clutter_get_script_id(G_OBJECT(actor->priv->template)), clutter_get_script_id(G_OBJECT(actor)));
-		} else {
-			if (tangle_template_are_parameters_complete(actor->priv->template)) {
-				tangle_template_apply_properties(actor->priv->template, G_OBJECT(actor));
-				TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->template);
-			}
-
-			retvalue = TRUE;
-		}
-	} else if (name[0] == '@') {
-		if (!clutter_script_parse_node(script, value, name, node, NULL)) {
-			g_warning("The value of a signal handler property '%s' could not be parsed (object: %s), skipped.", name + 1, clutter_get_script_id(G_OBJECT(actor)));
-		} else if (!G_VALUE_HOLDS_STRING(value)) {
-			g_warning("The value of a signal handler property '%s' of an object (id: %s) was not string, skipped.", name + 1, clutter_get_script_id(G_OBJECT(actor)));
-		} else if (!tangle_signal_connect_dynamically(G_OBJECT(actor), name + 1, g_value_get_string(value), NULL)) {
-			g_warning("Failed to connect signal handler property '%s' of an object (id: %s).", name + 1, clutter_get_script_id(G_OBJECT(actor)));
-		} else {
-			retvalue = TRUE;
-		}
-	} else if ((param_spec = g_object_class_find_property(G_OBJECT_CLASS(type_class), name)) &&
-	           G_IS_PARAM_SPEC_OBJECT(param_spec) &&
-		   !g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(param_spec), TANGLE_TYPE_TEMPLATE) &&
-	           (s = json_node_get_string(node)) &&
-		   (object = clutter_script_get_object(script, s)) &&
-		   TANGLE_IS_TEMPLATE(object)) {
-		if (!g_type_is_a(tangle_template_get_object_type(TANGLE_TEMPLATE(object)), G_PARAM_SPEC_VALUE_TYPE(param_spec))) {
-			g_warning("The type of the template (id: %s) does not match to property (name: %s), skipped.", s, name);
-		} else if (!(object = tangle_template_create_object(TANGLE_TEMPLATE(object)))) {
-			g_warning("Could not instantiate template (id: %s) for property (name: %s), skipped.", s, name);
-		} else {
-			g_value_init(value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
-			g_value_set_object(value, object);
-			
-			retvalue = TRUE;
-		}
-	} else if (parent_scriptable_iface->parse_custom_node) {
+	if (!(retvalue = _tangle_scriptable_parse_custom_node(scriptable, script, value, name, node, actor->priv->template, &actor->priv->template_applied)) &&
+	    parent_scriptable_iface->parse_custom_node) {
 		retvalue = parent_scriptable_iface->parse_custom_node(scriptable, script, value, name, node);
 	}
-
-	g_type_class_unref(type_class);
 	
 	return retvalue;
 }
 
 static void tangle_actor_set_custom_property(ClutterScriptable* scriptable, ClutterScript* script, const gchar* name, const GValue* value) {
-	if (name[0] != '$' && name[0] != '@') {
-		g_object_set_property(G_OBJECT(scriptable), name, value);
-	}
+	_tangle_scriptable_set_custom_property(scriptable, script, name, value);
 }
 
 static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
-	if (!(parent_scriptable_iface = parent_scriptable_iface = g_type_interface_peek_parent (iface))) {
+	if (!(parent_scriptable_iface = g_type_interface_peek_parent (iface))) {
 		parent_scriptable_iface = g_type_default_interface_peek(CLUTTER_TYPE_SCRIPTABLE);
 	}
 	
@@ -2312,9 +2246,8 @@ static gboolean check_template_parameters(gpointer user_data) {
 	
 	actor = TANGLE_ACTOR(user_data);
 	
-	if (actor->priv->template) {
+	if (actor->priv->template && !actor->priv->template_applied) {
 		g_critical("Some template (id: %s) parameters in actor (id: %s) are missing. Template is not used.", clutter_get_script_id(G_OBJECT(actor->priv->template)), clutter_get_script_id(G_OBJECT(actor)));
-		TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->template);
 	}
 	
 	return FALSE;
