@@ -51,6 +51,7 @@ struct _TangleButtonPrivate {
 static const GEnumValue button_mode_values[] = {
 	{ TANGLE_BUTTON_CLICKABLE, "TANGLE_BUTTON_CLICKABLE", "clickable" },
 	{ TANGLE_BUTTON_SELECTABLE, "TANGLE_BUTTON_SELECTABLE", "selectable" },
+	{ TANGLE_BUTTON_RADIO, "TANGLE_BUTTON_RADIO", "radio" },
 	{ 0, NULL, NULL }
 };
 
@@ -99,13 +100,28 @@ ClutterActor* tangle_button_new_selectable_with_background_actors(ClutterActor*
 	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_BUTTON, "button-mode", TANGLE_BUTTON_SELECTABLE, "normal-background-actor", normal_background_actor, "interactive-background-actor", interactive_background_actor, "selected-background-actor", selected_background_actor, NULL));
 }
 
+ClutterActor* tangle_button_new_radio(void) {
+
+	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_BUTTON, "button-mode", TANGLE_BUTTON_RADIO, NULL));
+}
+
+ClutterActor* tangle_button_new_radio_with_background_actor(ClutterActor* normal_background_actor) {
+
+	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_BUTTON, "button-mode", TANGLE_BUTTON_RADIO, "normal-background-actor", normal_background_actor, "prefer-background-size", TRUE, NULL));
+}
+
+ClutterActor* tangle_button_new_radio_with_background_actors(ClutterActor* normal_background_actor, ClutterActor* interactive_background_actor, ClutterActor* selected_background_actor) {
+
+	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_BUTTON, "button-mode", TANGLE_BUTTON_RADIO, "normal-background-actor", normal_background_actor, "interactive-background-actor", interactive_background_actor, "selected-background-actor", selected_background_actor, NULL));
+}
+
 TangleButtonMode tangle_button_get_mode(TangleButton* button) {
 
 	return button->priv->mode;
 }
 
 void tangle_button_set_mode(TangleButton* button, TangleButtonMode mode) {
-	g_return_if_fail(mode >= TANGLE_BUTTON_CLICKABLE && mode <= TANGLE_BUTTON_SELECTABLE);
+	g_return_if_fail(mode >= TANGLE_BUTTON_CLICKABLE && mode <= TANGLE_BUTTON_RADIO);
 	
 	if (button->priv->mode != mode) {
 		button->priv->mode = mode;
@@ -209,7 +225,7 @@ void tangle_button_set_selected_background_actor(TangleButton* button, ClutterAc
 gboolean tangle_button_get_selected(TangleButton* button) {
 	gboolean selected = FALSE;
 	
-	if (button->priv->mode == TANGLE_BUTTON_SELECTABLE) {
+	if ((button->priv->mode == TANGLE_BUTTON_SELECTABLE || button->priv->mode == TANGLE_BUTTON_RADIO)) {
 		selected = button->priv->selected;
 	}
 	
@@ -217,12 +233,13 @@ gboolean tangle_button_get_selected(TangleButton* button) {
 }
 	
 void tangle_button_set_selected(TangleButton* button, gboolean selected) {
-	g_return_if_fail(button->priv->mode == TANGLE_BUTTON_SELECTABLE);
+	g_return_if_fail((button->priv->mode == TANGLE_BUTTON_SELECTABLE || button->priv->mode == TANGLE_BUTTON_RADIO));
 	
 	if (button->priv->selected != selected) {
 		button->priv->selected = selected;
 		if (!button->priv->active) {
-			if (button->priv->mode == TANGLE_BUTTON_SELECTABLE && button->priv->selected && button->priv->selected_background_actor) {
+			if ((button->priv->mode == TANGLE_BUTTON_SELECTABLE || button->priv->mode == TANGLE_BUTTON_RADIO) &&
+			    button->priv->selected && button->priv->selected_background_actor) {
 				tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->selected_background_actor);
 			} else {
 				tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->normal_background_actor);
@@ -460,15 +477,11 @@ static void end_interacting(TangleButton* button) {
 	
 	tangle_actor_set_interacting(TANGLE_ACTOR(button), FALSE);
 	tangle_actor_send_interaction_ended_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
-	if (button->priv->mode == TANGLE_BUTTON_SELECTABLE && button->priv->selected && button->priv->selected_background_actor) {
+	if ((button->priv->mode == TANGLE_BUTTON_SELECTABLE || button->priv->mode == TANGLE_BUTTON_RADIO) && button->priv->selected && button->priv->selected_background_actor) {
 		tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->selected_background_actor);
 	} else {
 		tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->normal_background_actor);
 	}
-	if (button->priv->button_release_event_handler_id) {
-		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), button->priv->button_release_event_handler_id);
-		button->priv->button_release_event_handler_id = 0;
-	}
 
 	tangle_actor_send_interaction_ended_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
 }
@@ -479,8 +492,14 @@ static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event
 
 	button = TANGLE_BUTTON(user_data);
 
-	if (button->priv->inside && button->priv->mode == TANGLE_BUTTON_SELECTABLE) {
-		button->priv->selected = !button->priv->selected;
+	if (button->priv->inside) {
+		if (button->priv->mode == TANGLE_BUTTON_SELECTABLE) {
+			button->priv->selected = !button->priv->selected;
+			g_object_notify(G_OBJECT(button), "selected");
+		} else if (button->priv->mode == TANGLE_BUTTON_RADIO && !button->priv->selected) {
+			button->priv->selected = TRUE;
+			g_object_notify(G_OBJECT(button), "selected");
+		}
 	}
 
 	end_interacting(button);
@@ -490,6 +509,11 @@ static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event
 		g_signal_emit(button, signals[CLICKED], 0, &handled);
 	}
 
+	if (button->priv->button_release_event_handler_id) {
+		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), button->priv->button_release_event_handler_id);
+		button->priv->button_release_event_handler_id = 0;
+	}
+
 	return FALSE;
 }
 
