@@ -33,14 +33,14 @@ struct _TangleCompoundStylePrivate {
 static void add_style_property_reference(const gchar* name, const GValue* value, gpointer user_data);
 static void remove_style_property_reference(const gchar* name, const GValue* value, gpointer user_data);
 
-TangleCompoundStyle* tangle_compound_style_new(void) {
+TangleCompoundStyle* tangle_compound_style_new(GType for_type) {
 
-	return TANGLE_COMPOUND_STYLE(g_object_new(TANGLE_TYPE_COMPOUND_STYLE, NULL));
+	return TANGLE_COMPOUND_STYLE(g_object_new(TANGLE_TYPE_COMPOUND_STYLE, "for-type", for_type, NULL));
 }
 
-TangleCompoundStyle* tangle_compound_style_new_with_stylesheet(TangleStylesheet* stylesheet) {
+TangleCompoundStyle* tangle_compound_style_new_with_stylesheet(GType for_type, TangleStylesheet* stylesheet) {
 
-	return TANGLE_COMPOUND_STYLE(g_object_new(TANGLE_TYPE_COMPOUND_STYLE, "stylesheet", stylesheet, NULL));
+	return TANGLE_COMPOUND_STYLE(g_object_new(TANGLE_TYPE_COMPOUND_STYLE, "for-type", for_type, "stylesheet", stylesheet, NULL));
 }
 
 void tangle_compound_style_add_style(TangleCompoundStyle* compound_style, TangleStyle* style) {
