@@ -49,4 +49,7 @@ TangleEvent* tangle_event_new(TangleEventType type, TangleActor* sender);
 TangleEvent* tangle_event_copy(const TangleEvent* event);
 void tangle_event_free(TangleEvent* event);
 
+void tangle_actor_send_interaction_began_event(TangleActor* actor, gboolean x_axis, gboolean y_axis, gboolean dragging);
+void tangle_actor_send_interaction_ended_event(TangleActor* actor, gboolean x_axis, gboolean y_axis, gboolean dragging);
+
 #endif
