@@ -10,6 +10,7 @@
 
 #include "tangle-misc.h"
 #include "tangle-stylesheet.h"
+#include "tangle-vault.h"
 #include <gobject/gvaluecollector.h>
 #include <string.h>
 #include <clutter/clutter.h>
@@ -21,6 +22,8 @@
  * @Title: Miscellaneous utilities
  */
 
+static void on_weak_notify(gpointer user_data, GObject* other_object);
+
 gboolean tangle_init(int* argc, char*** argv) {
 	gboolean success = FALSE;
 	TangleStylesheet* stylesheet;
@@ -204,10 +207,15 @@ gboolean tangle_signal_accumulator_non_null_handled(GSignalInvocationHint* ihint
 	return !object;
 }
 
-gboolean tangle_signal_connect_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, gpointer user_data) {
+static gboolean connect_signal_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, gpointer user_data, gboolean swapped, gboolean set_weak_notify) {
 	gboolean retvalue = FALSE;
 	GModule* module = NULL;
 	gpointer function_pointer;
+	TangleVault* vault;
+
+	g_return_val_if_fail(G_IS_OBJECT(object), FALSE);	
+	g_return_val_if_fail(signal_name != NULL, FALSE);
+	g_return_val_if_fail(function_name != NULL, FALSE);
 	
 	if (!g_signal_parse_name(signal_name, G_TYPE_FROM_INSTANCE(object), NULL, NULL, FALSE)) {
 		g_warning("No such signal: '%s'.", signal_name);
@@ -215,10 +223,19 @@ gboolean tangle_signal_connect_dynamically(GObject* object, const gchar* signal_
 		g_warning("The current platform does not support modules.");
 	} else if (!g_module_symbol(module, function_name, (gpointer*)&function_pointer)) {
 		g_warning("No such function: '%s'", function_name);
-	} else if (!g_signal_connect(object, signal_name, G_CALLBACK(function_pointer), user_data)) {
-		g_warning("Failed to connect a handler function '%s' for signal '%s'.", function_name, signal_name);
 	} else {
-		retvalue = TRUE;
+		if (swapped) {
+			retvalue = g_signal_connect_swapped(object, signal_name, G_CALLBACK(function_pointer), user_data) != 0;
+			if (set_weak_notify) {
+				vault = tangle_vault_new(2, G_TYPE_OBJECT, object, G_TYPE_POINTER, function_pointer);
+				g_object_weak_ref(G_OBJECT(user_data), on_weak_notify, vault);
+			}
+		} else {
+			retvalue = g_signal_connect(object, signal_name, G_CALLBACK(function_pointer), user_data) != 0;
+		}
+		if (!retvalue) {
+			g_warning("Failed to connect a handler function '%s' for signal '%s'.", function_name, signal_name);
+		}
 	}
 	if (module) {
 		g_module_close(module);
@@ -226,3 +243,53 @@ gboolean tangle_signal_connect_dynamically(GObject* object, const gchar* signal_
 	
 	return retvalue;
 }
+
+gboolean tangle_signal_connect_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, gpointer user_data) {
+
+	return connect_signal_dynamically(object, signal_name, function_name, user_data, FALSE, FALSE);
+}
+
+gboolean tangle_signal_connect_swapped_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, gpointer user_data) {
+
+	return connect_signal_dynamically(object, signal_name, function_name, user_data, TRUE, FALSE);
+}
+
+gboolean tangle_signal_connect_object_swapped_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, GObject* other_object) {
+
+	return connect_signal_dynamically(object, signal_name, function_name, other_object, TRUE, TRUE);
+}
+
+gboolean tangle_signal_connect_dynamically_from_script(GObject* object, const gchar* signal_name, const gchar* function_name, ClutterScript* script) {
+	gboolean retvalue = FALSE;
+	const gchar* colon;
+	gchar* id;
+	GObject* other_object;
+
+	g_return_val_if_fail(CLUTTER_IS_SCRIPT(script), FALSE);
+	
+	if ((colon = strchr(signal_name, ':')) && colon != signal_name && colon[1] != 0 && colon[1] != ':') {
+		id = g_strndup(signal_name, colon - signal_name);
+		if ((other_object = clutter_script_get_object(script, id)) &&
+		    tangle_signal_connect_object_swapped_dynamically(other_object, colon + 1, function_name, object)) {
+			retvalue = TRUE;
+		}			
+		g_free(id);
+	} else if (tangle_signal_connect_dynamically(object, signal_name, function_name, NULL)) {
+		retvalue = TRUE;
+	}
+	
+	return retvalue;
+}
+
+static void on_weak_notify(gpointer user_data, GObject* other_object) {
+	TangleVault* vault;
+	GObject* object;
+	gpointer function_pointer;
+	
+	vault = TANGLE_VAULT(user_data);
+	tangle_vault_get(vault, 2, G_TYPE_OBJECT, &object, G_TYPE_POINTER, &function_pointer);
+	
+	g_signal_handlers_disconnect_by_func(object, G_CALLBACK(function_pointer), other_object);
+	
+	tangle_vault_free(vault);
+}
