@@ -130,20 +130,25 @@ void tangle_properties_set_property(TangleProperties* properties, const gchar* n
 	g_signal_emit(properties, signals[CHANGED], 0, name, value);
 }
 
-void tangle_properties_set(TangleProperties* properties, const gchar* first_name, GType first_type, ...) {
+void tangle_properties_set(TangleProperties* properties, const gchar* first_name, ...) {
+	va_list properties_list;
+
+	va_start(properties_list, properties);
+	tangle_properties_set_valist(properties, properties_list);
+	va_end(properties_list);
+}
+
+void tangle_properties_set_valist(TangleProperties* properties, va_list properties_list) {
 	const gchar* name;
 	GType type;
-	va_list args;
 	GValue value = { 0 };
 	gchar* error;
-	
-	name = first_name;
-	type = first_type;
-	va_start(args, first_type);
-	do {
+
+	while ((name = va_arg(properties_list, const gchar*))) {
+		type = va_arg(properties_list, GType);
 		g_value_init(&value, type);
 		error = NULL;
-		G_VALUE_COLLECT(&value, args, 0, &error);
+		G_VALUE_COLLECT(&value, properties_list, 0, &error);
 		if (error) {
 			g_warning("%s: %s", G_STRLOC, error);
 			g_free(error);
@@ -151,12 +156,7 @@ void tangle_properties_set(TangleProperties* properties, const gchar* first_name
 			tangle_properties_set_property(properties, name, &value);
 		}
 		g_value_unset(&value);
-		
-		name = va_arg(args, const gchar*);
-		if (name) {
-			type = va_arg(args, GType);
-		}
-	} while (name);	
+	}	
 }
 
 void tangle_properties_foreach(TangleProperties* properties, TanglePropertyCallback callback, gpointer user_data) {
