@@ -129,10 +129,6 @@ static void tangle_show_actor_action_get_property(GObject* object, guint prop_id
         }
 }
 
-static void tangle_show_actor_action_finalize(GObject* object) {
-	G_OBJECT_CLASS(tangle_show_actor_action_parent_class)->finalize(object);
-}
-
 static void tangle_show_actor_action_dispose(GObject* object) {
 	TangleShowActorAction* action;
 	
@@ -147,7 +143,6 @@ static void tangle_show_actor_action_class_init(TangleShowActorActionClass* klas
 	GObjectClass* gobject_class = G_OBJECT_CLASS(klass);
 	TangleActionClass* action_class = TANGLE_ACTION_CLASS(klass);
 
-	gobject_class->finalize = tangle_show_actor_action_finalize;
 	gobject_class->dispose = tangle_show_actor_action_dispose;
 	gobject_class->set_property = tangle_show_actor_action_set_property;
 	gobject_class->get_property = tangle_show_actor_action_get_property;
