@@ -12,9 +12,13 @@
 #include "tangle-stylable.h"
 #include "tangle-template.h"
 #include "tangle-action.h"
+#include "tangle-vault.h"
 #include "tangle-misc.h"
 #include "marshalers.h"
 
+/* We know that this is available in the Clutter library (from clutter-script-private.h). */
+GType clutter_script_get_type_from_class (const gchar *name);
+
 /**
  * SECTION:tangle-style
  * @Short_description: Style information related to a specied class of objects
@@ -28,7 +32,9 @@ G_DEFINE_TYPE_WITH_CODE(TangleStyle, tangle_style, G_TYPE_OBJECT,
 
 enum {
 	PROP_0,
+	PROP_FOR_TYPE,
 	PROP_FOR_CLASS,
+	PROP_FOR_SUBCLASSES,
 	PROP_FOR_NAME
 };
 
@@ -38,9 +44,12 @@ enum {
 };
 
 struct _TangleStylePrivate {
-	gchar* for_class;
+	GType for_type;
 	gchar* for_name;
 	TangleProperties* style_properties;
+	ClutterScript* script;
+	
+	guint for_subclasses : 1;
 };
 
 static GQuark quark_object_style = 0;
@@ -52,25 +61,26 @@ static void unset_style_property(const gchar* name, const GValue* value, gpointe
 static void on_notify(GObject* object, GParamSpec* param_spec, gpointer user_data);
 static void on_style_changed(TangleStyle* style, const gchar* name, const GValue* old_value, const GValue* new_value, gpointer user_data);
 static void disconnect_style_changed_handler(gpointer user_data, GObject* object);
-static void set_object_property(GObject* object, const gchar* name, const GValue* value);
+static void set_object_property(GObject* object, const gchar* name, GType type, const GValue* value);
+static void handle_object_reference(TangleStyle* style, const gchar* name, GValue* value);
+static void foreach_style_property(const gchar* name, const GValue* value, gpointer user_data);
 
 #define BLOCK_NOTIFY_HANDLER(o,h) g_signal_handlers_block_by_func(o, G_CALLBACK(on_notify), h);
 #define UNBLOCK_NOTIFY_HANDLER(o,h) g_signal_handlers_unblock_by_func(o, G_CALLBACK(on_notify), h);
 
-TangleStyle* tangle_style_new(void) {
+TangleStyle* tangle_style_new(GType for_type) {
 
-	return TANGLE_STYLE(g_object_new(TANGLE_TYPE_STYLE, NULL));
+	return TANGLE_STYLE(g_object_new(TANGLE_TYPE_STYLE, "for-type", for_type, NULL));
+}
+
+GType tangle_style_get_for_type(TangleStyle* style) {
+
+	return style->priv->for_type;
 }
 
 const gchar* tangle_style_get_for_class(TangleStyle* style) {
 
-	return style->priv->for_class;
-}
-
-void tangle_style_set_for_class(TangleStyle* style, const gchar* for_class) {
-	g_free(style->priv->for_class);
-	style->priv->for_class = g_strdup(for_class);
-	g_object_notify(G_OBJECT(style), "for-class");
+	return g_type_name(style->priv->for_type);
 }
 
 const gchar* tangle_style_get_for_name(TangleStyle* style) {
@@ -84,6 +94,18 @@ void tangle_style_set_for_name(TangleStyle* style, const gchar* for_name) {
 	g_object_notify(G_OBJECT(style), "for-name");
 }
 
+gboolean tangle_style_get_for_subclasses(TangleStyle* style) {
+
+	return style->priv->for_subclasses;
+}
+
+void tangle_style_set_for_subclasses(TangleStyle* style, gboolean for_subclasses) {
+	if (style->priv->for_subclasses != for_subclasses) {
+		style->priv->for_subclasses = for_subclasses;
+		g_object_notify(G_OBJECT(style), "for-subclasses");
+	}
+}
+
 gboolean tangle_style_get_style_property(TangleStyle* style, const gchar* name, GValue* value) {
 
 	return TANGLE_STYLE_GET_CLASS(style)->get_style_property(style, name, value);
@@ -100,10 +122,9 @@ void tangle_style_foreach_style_property(TangleStyle* style, TanglePropertyCallb
 }
 
 gboolean tangle_style_is_for_object(TangleStyle* style, GObject* object) {
-	/* TODO: support parent classes and parent:name style... */
 	
-	return (!style->priv->for_class || 
-	        !strcmp(style->priv->for_class, G_OBJECT_TYPE_NAME(object))) &&
+	return ((style->priv->for_subclasses && g_type_is_a(G_OBJECT_TYPE(object), style->priv->for_type)) ||
+	        (!style->priv->for_subclasses && G_OBJECT_TYPE(object) == style->priv->for_type)) &&
 	       (!CLUTTER_IS_ACTOR(object) ||
 	        !style->priv->for_name ||
 		(clutter_actor_get_name(CLUTTER_ACTOR(object)) &&
@@ -127,8 +148,8 @@ void tangle_style_apply(TangleStyle* style, GObject* object) {
 		g_object_set_qdata_full(object, quark_object_style_properties, hash_table, (GDestroyNotify)g_hash_table_unref);
 		g_signal_connect(object, "notify", G_CALLBACK(on_notify), hash_table);
 
-		tangle_style_foreach_style_property(style, set_style_property, object);
 		g_object_set_qdata(object, quark_object_style, style);
+		tangle_style_foreach_style_property(style, set_style_property, object);
 				
 		g_signal_connect(style, "style-changed", G_CALLBACK(on_style_changed), object);
 		g_object_weak_ref(object, disconnect_style_changed_handler, style);
@@ -175,8 +196,13 @@ gboolean tangle_object_is_style_property(GObject* object, const gchar* name) {
 }
 
 static gboolean tangle_style_get_style_property_impl(TangleStyle* style, const gchar* name, GValue* value) {
+	gboolean retvalue;
+	
+	if (retvalue = tangle_properties_get_property(style->priv->style_properties, name, value)) {
+		handle_object_reference(style, name, value);
+	}
 
-	return tangle_properties_get_property(style->priv->style_properties, name, value);
+	return retvalue;
 }
 
 static void tangle_style_set_style_property_impl(TangleStyle* style, const gchar* name, const GValue* value) {
@@ -188,19 +214,45 @@ static void tangle_style_set_style_property_impl(TangleStyle* style, const gchar
 }
 
 static void tangle_style_foreach_style_property_impl(TangleStyle* style, TanglePropertyCallback callback, gpointer user_data) {
+	TangleVault* vault;
+	
+	vault = tangle_vault_new(3, TANGLE_TYPE_STYLE, style, G_TYPE_POINTER, callback, G_TYPE_POINTER, user_data);
 
-	return tangle_properties_foreach(style->priv->style_properties, callback, user_data);
+	return tangle_properties_foreach(style->priv->style_properties, foreach_style_property, vault);
 }
 
 static void tangle_style_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
 	TangleStyle* style;
+	GType type;
+	const gchar* string;
 	
 	style = TANGLE_STYLE(object);
 
 	switch (prop_id) {
+		case PROP_FOR_TYPE:
+			if ((type = g_value_get_gtype(value)) != G_TYPE_NONE) {
+				if (style->priv->for_type != G_TYPE_NONE) {
+					g_warning("Trying to set a for-type of the style that already has it, ignored.");
+				} else {
+					style->priv->for_type = type;
+				}
+			}
+			break;	
 		case PROP_FOR_CLASS:
-			tangle_style_set_for_class(style, g_value_get_string(value));
+			if ((string = g_value_get_string(value))) {
+				if (style->priv->for_type != G_TYPE_NONE) {
+					g_warning("Trying to set a for-class of the style that already has for-type, ignored.");
+				} else if ((type = g_type_from_name(string)) != G_TYPE_INVALID) {
+					style->priv->for_type = type;
+				} else if ((type = clutter_script_get_type_from_class(string)) != G_TYPE_INVALID) {
+					style->priv->for_type = type;
+				} else {
+					g_warning("No such class: %s.", string);
+				}
+			}
 			break;
+		case PROP_FOR_SUBCLASSES:
+			tangle_style_set_for_subclasses(style, g_value_get_boolean(value));
 		case PROP_FOR_NAME:
 			tangle_style_set_for_name(style, g_value_get_string(value));
 			break;
@@ -216,11 +268,16 @@ static void tangle_style_get_property(GObject* object, guint prop_id, GValue* va
 	style = TANGLE_STYLE(object);
 
         switch (prop_id) {
+		case PROP_FOR_TYPE:
+			g_value_set_gtype(value, style->priv->for_type);
+			break;
 		case PROP_FOR_CLASS:
 			g_value_set_string(value, tangle_style_get_for_class(style));
 			break;
+		case PROP_FOR_SUBCLASSES:
+			g_value_set_boolean(value, style->priv->for_subclasses);
 		case PROP_FOR_NAME:
-			g_value_set_string(value, tangle_style_get_for_name(style));
+			g_value_set_string(value, style->priv->for_name);
 			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -228,12 +285,21 @@ static void tangle_style_get_property(GObject* object, guint prop_id, GValue* va
         }
 }
 
+static void tangle_style_constructed(GObject* object) {
+	TangleStyle* style;
+	
+	style = TANGLE_STYLE(object);
+	
+	if (style->priv->for_type == G_TYPE_NONE) {
+		g_critical("No valid for-type or for-class provided for style.");
+	}
+}
+
 static void tangle_style_finalize(GObject* object) {
 	TangleStyle* style;
 	
 	style = TANGLE_STYLE(object);
 
-	g_free(style->priv->for_class);
 	g_free(style->priv->for_name);
 
 	G_OBJECT_CLASS(tangle_style_parent_class)->finalize(object);
@@ -256,11 +322,21 @@ static void tangle_style_class_init(TangleStyleClass* style_class) {
 	style_class->set_style_property = tangle_style_set_style_property_impl;
 	style_class->foreach_style_property = tangle_style_foreach_style_property_impl;
 
+	gobject_class->constructed = tangle_style_constructed;
 	gobject_class->finalize = tangle_style_finalize;
 	gobject_class->dispose = tangle_style_dispose;
 	gobject_class->set_property = tangle_style_set_property;
 	gobject_class->get_property = tangle_style_get_property;
 
+	/**
+	 * TangleStyle:for-type:
+	 */
+	g_object_class_install_property(gobject_class, PROP_FOR_TYPE,
+	                                g_param_spec_gtype("for-type",
+	                                "For type",
+	                                "The type of the class that is target of this style",
+	                                G_TYPE_NONE,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleStyle:for-class:
 	 */
@@ -269,6 +345,15 @@ static void tangle_style_class_init(TangleStyleClass* style_class) {
 	                                "For class",
 	                                "The name of the class that is target of this style",
 	                                NULL,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleStyle:for-subclasses:
+	 */
+	g_object_class_install_property(gobject_class, PROP_FOR_SUBCLASSES,
+	                                g_param_spec_boolean("for-subclasses",
+	                                "For subclasses",
+	                                "Whether the style is applied to subclasses of the for-type/for-class",
+	                                TRUE,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleStyle:for-name:
@@ -298,13 +383,24 @@ static void tangle_style_class_init(TangleStyleClass* style_class) {
 
 static void tangle_style_init(TangleStyle* style) {
 	style->priv = G_TYPE_INSTANCE_GET_PRIVATE(style, TANGLE_TYPE_STYLE, TangleStylePrivate);
-	style->priv->style_properties = tangle_properties_new();	
+	style->priv->style_properties = tangle_properties_new();
+	style->priv->for_type = G_TYPE_NONE;
+	style->priv->for_subclasses = TRUE;	
 }
 
 static void tangle_style_set_custom_property(ClutterScriptable* scriptable, ClutterScript* script,  const gchar* name, const GValue* value) {
 	TangleStyle* style;
 
 	style = TANGLE_STYLE(scriptable);
+
+	g_assert(!style->priv->script || style->priv->script == script);
+
+	if (!style->priv->script) {
+		/* TODO: This should be weak, should it not? */
+		g_object_ref(script);
+		style->priv->script = script;
+	}
+
 	if (g_object_class_find_property(G_OBJECT_CLASS(TANGLE_STYLE_GET_CLASS(style)), name)) {
 		g_object_set_property(G_OBJECT(style), name, value);
 	} else {
@@ -336,7 +432,7 @@ static void set_style_property(const gchar* name, const GValue* value, gpointer
 			g_object_get_property(object, name, &current_value);
 			if (g_param_value_defaults(param_spec, &current_value)) {
 				BLOCK_NOTIFY_HANDLER(object, hash_table);
-				set_object_property(object, name, value);
+				set_object_property(object, name, G_PARAM_SPEC_VALUE_TYPE(param_spec), value);
 				UNBLOCK_NOTIFY_HANDLER(object, hash_table);
 				g_hash_table_insert(hash_table, g_strdup(name), NULL);
 			}
@@ -367,7 +463,7 @@ static void unset_style_property(const gchar* name, const GValue* value, gpointe
 		} else if (param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), name)) {
 			g_value_init(&current_value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
 			g_param_value_set_default(param_spec, &current_value);
-			set_object_property(object, name, &current_value);
+			set_object_property(object, name, G_PARAM_SPEC_VALUE_TYPE(param_spec), &current_value);
 			g_value_unset(&current_value);
 		} else {
 			g_warning("%s: object does not have property called '%s'.", G_STRLOC, name);
@@ -392,8 +488,10 @@ static void change_style_property(GObject* object, const gchar* name, const GVal
 		if (G_VALUE_HOLDS(old_value, TANGLE_TYPE_ACTION)) {
 			tangle_action_remove(TANGLE_ACTION(g_value_get_object(old_value)), object, name);
 			tangle_action_add(TANGLE_ACTION(g_value_get_object(new_value)), object, name);
+		} else if (param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), name)) {
+			set_object_property(object, name, G_PARAM_SPEC_VALUE_TYPE(param_spec), new_value);
 		} else {
-			set_object_property(object, name, new_value);
+			g_warning("%s: object does not have property called '%s'.", G_STRLOC, name);
 		}
 		UNBLOCK_NOTIFY_HANDLER(object, hash_table);
 	}
@@ -427,13 +525,13 @@ static void disconnect_style_changed_handler(gpointer user_data, GObject* object
 	g_signal_handlers_disconnect_by_func(style, G_CALLBACK(on_style_changed), object);
 }
 
-static void set_object_property(GObject* object, const gchar* name, const GValue* value) {
+static void set_object_property(GObject* object, const gchar* name, GType type, const GValue* value) {
 	TangleTemplate* template;
 	GObject* value_object;
 	GValue object_value = { 0 };
 	TangleAction* action;
 	
-	if (G_VALUE_HOLDS(value, TANGLE_TYPE_TEMPLATE)) {
+	if (G_VALUE_HOLDS(value, TANGLE_TYPE_TEMPLATE) && !g_type_is_a(type, TANGLE_TYPE_TEMPLATE)) {
 		template = TANGLE_TEMPLATE(g_value_get_object(value));
 		value_object = tangle_template_create_object(template);
 		g_object_ref_sink(value_object);
@@ -446,3 +544,42 @@ static void set_object_property(GObject* object, const gchar* name, const GValue
 		g_object_set_property(object, name, value);
 	}
 }
+
+static void handle_object_reference(TangleStyle* style, const gchar* name, GValue* value) {
+	gpointer type_class;
+	GParamSpec* param_spec;
+	GObject* object;
+
+	if (G_VALUE_HOLDS_STRING(value) &&
+	    style->priv->script &&
+	    style->priv->for_type != G_TYPE_NONE) {
+		type_class = g_type_class_ref(style->priv->for_type);
+
+		if ((param_spec = g_object_class_find_property(G_OBJECT_CLASS(type_class), name)) &&
+		    G_IS_PARAM_SPEC_OBJECT(param_spec) &&
+		    (object = clutter_script_get_object(style->priv->script, g_value_get_string(value)))) {
+			g_value_unset(value);
+			g_value_init(value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
+			g_value_set_instance(value, object);
+
+			tangle_properties_set_property(style->priv->style_properties, name, value);
+		}
+
+		g_type_class_unref(type_class);
+	}
+}
+
+static void foreach_style_property(const gchar* name, const GValue* value, gpointer user_data) {
+	TangleVault* vault;
+	TangleStyle* style;
+	gpointer callback;
+	gpointer callback_data;
+	
+	vault = TANGLE_VAULT(user_data);
+	tangle_vault_get(vault, 3, TANGLE_TYPE_STYLE, &style, G_TYPE_POINTER, &callback, G_TYPE_POINTER, &callback_data);
+	
+	/* We can omit the constness of value type because we actually own it. */
+	handle_object_reference(style, name, (GValue*)value);
+	
+	TANGLE_PROPERTY_CALLBACK(callback)(name, value, callback_data);
+}
