@@ -40,6 +40,7 @@ struct _TangleTemplatePrivate {
 	GHashTable* parameters;
 	
 	ClutterScript* script;
+	guint instance_counter;
 };
 
 static gboolean is_value_null(gpointer key, gpointer value, gpointer user_data);
@@ -67,34 +68,84 @@ GObject* tangle_template_create_object(TangleTemplate* template) {
 	gboolean error = FALSE;
 	guint n;
 	GParameter* parameters;
-	GArray* signals;
-	guint i, k;
+	GArray* construct_parameters;
+	GArray* nonconstruct_parameters;
+	GArray* signal_parameters;
+	gpointer type_class;
+	guint i;
+	GParamSpec* param_spec;
+	const gchar* template_id;
+	gchar* id;
+	ClutterScriptable* scriptable;
+	ClutterScriptableIface* scriptable_iface;
+	GParameter* parameter;
 
 	parameters = tangle_properties_get_as_parameters(TANGLE_PROPERTIES(template), &n);
-	signals = g_array_new(FALSE, FALSE, sizeof(GParameter));
+	construct_parameters = g_array_new(FALSE, FALSE, sizeof(GParameter));
+	nonconstruct_parameters = g_array_new(FALSE, FALSE, sizeof(GParameter));
+	signal_parameters = g_array_new(FALSE, FALSE, sizeof(GParameter));
 
+	type_class = g_type_class_ref(template->priv->object_type);
 	for (i = 0; i < n; i++) {
 		if (parameters[i].name[0] == '@') {
-			g_array_append_val(signals, parameters[i]);
-			for (k = i + 1; k < n; k++) {
-				parameters[k - 1] = parameters[k];
-			}
-			n--;
-			i--;
+			g_array_append_val(signal_parameters, parameters[i]);
 		} else if (!apply_parameters(template, parameters[i].name, &parameters[i].value)) {
 			error = TRUE;
 			break;
+		} else if ((param_spec = g_object_class_find_property(type_class, parameters[i].name)) &&
+		           param_spec->flags & G_PARAM_CONSTRUCT_ONLY) {
+			g_array_append_val(construct_parameters, parameters[i]);
+		} else {
+			g_array_append_val(nonconstruct_parameters, parameters[i]);
 		}
 	}
+	g_type_class_unref(type_class);
 
 	if (!error) {
-		object = g_object_newv(template->priv->object_type, n, parameters);
+		object = g_object_newv(template->priv->object_type, construct_parameters->len, (GParameter*)construct_parameters->data);
 		
-		for (i = 0; i < signals->len; i++) {
-			if (!G_VALUE_HOLDS(&g_array_index(signals, GParameter, i).value, G_TYPE_STRING) || !substitute_parameters(template, &g_array_index(signals, GParameter, i).value)) {
-				g_warning("Failed to connect signal handler property '%s' of an object.", g_array_index(signals, GParameter, i).name + 1);
-			} else if (!tangle_signal_connect_dynamically(object, parameters[i].name + 1, g_value_get_string(&g_array_index(signals, GParameter, i).value), NULL)) {
-				g_warning("Failed to connect signal handler property '%s' of an object.", g_array_index(signals, GParameter, i).name + 1);
+		template->priv->instance_counter++;
+		if (template->priv->script) {
+			if ((template_id = clutter_get_script_id(G_OBJECT(template)))) {
+				id = g_strdup_printf("%s$%d", template_id, template->priv->instance_counter);
+				if (CLUTTER_IS_SCRIPTABLE(object)) {
+					clutter_scriptable_set_id(CLUTTER_SCRIPTABLE(object), id);
+					g_free(id);
+				} else {
+					g_object_set_data_full(object, "clutter-script-id", id, g_free);
+				}
+			}
+			if (TANGLE_IS_ACTOR(object)) {
+				_tangle_actor_set_script(TANGLE_ACTOR(object), template->priv->script);
+			}
+		}
+		
+		if (!template->priv->script ||
+		    !CLUTTER_IS_SCRIPTABLE(object) ||
+		    !(scriptable = CLUTTER_SCRIPTABLE(object)) ||
+		    !(scriptable_iface = CLUTTER_SCRIPTABLE_GET_IFACE(scriptable)) ||
+		    !scriptable_iface->set_custom_property) {
+		    	scriptable_iface = NULL;
+		}
+		for (i = 0; i < nonconstruct_parameters->len; i++) {
+			parameter = &g_array_index(nonconstruct_parameters, GParameter, i);
+			if (scriptable_iface) {
+				scriptable_iface->set_custom_property(scriptable, template->priv->script, parameter->name, &parameter->value);
+			} else {
+				g_object_set_property(object, parameter->name, &parameter->value);
+			}
+		}
+
+		for (i = 0; i < signal_parameters->len; i++) {
+			parameter = &g_array_index(signal_parameters, GParameter, i);
+			if (!G_VALUE_HOLDS(&parameter->value, G_TYPE_STRING) || !substitute_parameters(template, &parameter->value)) {
+				g_warning("Failed to connect the signal handler property '%s' of an object created from the template '%s', skipped.", parameter->name + 1, clutter_get_script_id(G_OBJECT(template)));
+			} else if (template->priv->script) {
+				if (!!tangle_signal_connect_dynamically_from_script(object, parameter[i].name + 1, g_value_get_string(&parameter->value), template->priv->script)) {
+					g_warning("Failed to connect the signal handler property '%s' of an object created from the template '%s', skipped.", parameters->name + 1, clutter_get_script_id(G_OBJECT(template)));
+				}
+			} else if (!tangle_signal_connect_dynamically(object, parameter[i].name + 1, g_value_get_string(&parameter->value), NULL)) {
+				g_warning("Failed to connect signal handler property '%s' of an object, skipped.", parameter->name + 1);
 			}		
 		}
 	}
@@ -104,13 +155,11 @@ GObject* tangle_template_create_object(TangleTemplate* template) {
 		g_value_unset(&parameters[i].value);
 	}
 	g_free(parameters);
-	
-	for (i = 0; i < signals->len; i++) {
-		g_free((gchar*)g_array_index(signals, GParameter, i).name);
-		g_value_unset(&g_array_index(signals, GParameter, i).value);
-	}
-	g_array_free(signals, TRUE);
-	
+
+	g_array_free(construct_parameters, TRUE);
+	g_array_free(nonconstruct_parameters, TRUE);
+	g_array_free(signal_parameters, TRUE);
+
 	return object;
 }
 
@@ -392,6 +441,7 @@ static gboolean tangle_template_parse_custom_node(ClutterScriptable* scriptable,
 
 		}
 	}
+	
 	return retvalue;
 }
 
@@ -500,9 +550,13 @@ static void set_property(const gchar* name, const GValue* value, gpointer user_d
 	g_value_copy(value, &v);
 	if (name[0] == '@') {
 		if (!G_VALUE_HOLDS(&v, G_TYPE_STRING) || !substitute_parameters(template, &v)) {
-			g_warning("Failed to connect signal handler property '%s' of an object (id: %s).", name + 1, clutter_get_script_id(G_OBJECT(object)));
+			g_warning("Failed to connect signal handler property '%s' of an object (id: %s).", name, clutter_get_script_id(G_OBJECT(object)));
+		} else if (template->priv->script) {
+			if (!tangle_signal_connect_dynamically_from_script(object, name + 1, g_value_get_string(&v), template->priv->script)) {
+				g_warning("Failed to connect the signal handler property '%s' of an object created from the template '%s', skipped.", name + 1, clutter_get_script_id(G_OBJECT(template)));
+			}
 		} else if (!tangle_signal_connect_dynamically(object, name + 1, g_value_get_string(&v), NULL)) {
-			g_warning("Failed to connect signal handler property '%s' of an object (id: %s).", name + 1, clutter_get_script_id(G_OBJECT(object)));
+			g_warning("Failed to connect the signal handler property '%s' of an object, skipped.", name + 1);
 		}
 	} else if (apply_parameters(template, name, &v)) {
 		if (!mode || !duration || !CLUTTER_IS_ACTOR(object)) {
