@@ -17,9 +17,11 @@
  * @Title: TangleWrapperActor
  */
 
+static void clutter_container_iface_init(ClutterContainerIface *iface);
 static void clutter_scriptable_iface_init(ClutterScriptableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE(TangleWrapperActor, tangle_wrapper_actor, TANGLE_TYPE_ACTOR,
+                        G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_CONTAINER, clutter_container_iface_init);
                         G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_SCRIPTABLE, clutter_scriptable_iface_init););
 
 enum {
@@ -298,6 +300,41 @@ static void tangle_wrapper_actor_init(TangleWrapperActor* wrapper_actor) {
 	wrapper_actor->priv = G_TYPE_INSTANCE_GET_PRIVATE(wrapper_actor, TANGLE_TYPE_WRAPPER_ACTOR, TangleWrapperActorPrivate);
 }
 
+static void tangle_wrapper_actor_add(ClutterContainer* container, ClutterActor* actor) {	
+	g_critical("TangleWrapperActor does not support clutter_container_add().");
+}
+
+static void tangle_wrapper_actor_remove(ClutterContainer* container, ClutterActor* actor) {
+	g_critical("TangleWrapperActor does not support clutter_container_remove().");
+}
+
+static void tangle_wrapper_actor_foreach(ClutterContainer* container, ClutterCallback callback, gpointer user_data) {
+	g_return_if_fail(TANGLE_IS_WRAPPER_ACTOR(container));
+
+	callback(TANGLE_WRAPPER_ACTOR(container)->priv->wrapped, user_data);
+}
+
+static void tangle_wrapper_actor_raise(ClutterContainer* container, ClutterActor* actor, ClutterActor* sibling) {
+	g_return_if_fail(TANGLE_IS_WRAPPER_ACTOR(container));
+}
+
+static void tangle_wrapper_actor_lower(ClutterContainer* container, ClutterActor* actor, ClutterActor* sibling) {
+	g_return_if_fail(TANGLE_IS_WRAPPER_ACTOR(container));
+}
+
+static void tangle_wrapper_actor_sort_depth_order(ClutterContainer* container) {
+	g_return_if_fail(TANGLE_IS_WRAPPER_ACTOR(container));
+}
+
+static void clutter_container_iface_init(ClutterContainerIface* iface) {
+	iface->add = tangle_wrapper_actor_add;
+	iface->remove = tangle_wrapper_actor_remove;
+	iface->foreach = tangle_wrapper_actor_foreach;
+	iface->raise = tangle_wrapper_actor_raise;
+	iface->lower = tangle_wrapper_actor_lower;
+	iface->sort_depth_order = tangle_wrapper_actor_sort_depth_order;
+}
+
 static void tangle_wrapper_actor_set_custom_property(ClutterScriptable* scriptable, ClutterScript* script,  const gchar* name, const GValue* value) {
 	TangleWrapperActor* wrapper_actor;
 	GParamSpec* param_spec;
